import pandas as pd

basic_text = "Not defined, it should to allow user to connect"
key_text = "Connect key missing"
basic_error = "Not defined, it should return a Dataframe"
connect_error = "You should connect first"


class ConnectDriver:

    connected = False
    key = None
    raise_error = False

    def open_or_read(self, data):
        read_data = data
        try:
            read_data = open(data, "r").read()
        except OSError:
            pass
        return read_data

    def print_error(self, error):
        if self.raise_error:
            raise ValueError(error)
        else:
            print(error)

    def raise_for_error(self, raise_error=True):
        self.raise_error = raise_error

    def connect(self, key=None):
        self.key = key
        if self.key:
            self.connected = True
        else:
            self.print_error(key_text)
        return self

    def check_connect(self):
        if not self.connected:
            self.print_error(connect_error)


class InDriver(ConnectDriver):
    def convert_data_to_df(self, *args, **kwargs):
        return basic_error

    def get(self, *args, **kwargs) -> pd.DataFrame:
        self.check_connect()
        self.print_error(basic_error)


class OutDriver(ConnectDriver):
    def send(self, *args, **kwargs):
        self.check_connect()
        self.print_error(basic_error)
        return basic_error



# flake8: noqa

from typing import Optional, Union, Dict, Any

from transformers import (
    PretrainedConfig,
    PreTrainedTokenizer,
    AutoConfig,
    AutoTokenizer,
)

from naas_drivers.driver import InDriver
from transformers.pipelines.question_answering import QuestionAnsweringPipeline
from transformers.pipelines.text2text_generation import (
    SummarizationPipeline,
    Text2TextGenerationPipeline,
    TranslationPipeline,
)
from transformers.pipelines.text_classification import TextClassificationPipeline
from transformers.pipelines.text_generation import TextGenerationPipeline
from transformers.pipelines.feature_extraction import FeatureExtractionPipeline
from transformers.pipelines.token_classification import TokenClassificationPipeline
from transformers.pipelines.fill_mask import FillMaskPipeline
from transformers.pipelines.table_question_answering import (
    TableQuestionAnsweringPipeline,
)
from transformers.pipelines.zero_shot_classification import (
    ZeroShotClassificationPipeline,
)
from transformers.pipelines.conversational import ConversationalPipeline
from transformers.pipelines.image_classification import ImageClassificationPipeline
from transformers.pipelines.base import infer_framework_from_model, Pipeline
from transformers.utils import logging
from transformers.file_utils import is_torch_available, is_tf_available


logger = logging.get_logger(__name__)

if is_torch_available():
    import torch
    from transformers.models.auto.modeling_auto import (
        AutoModelForSequenceClassification,
        AutoModelForSeq2SeqLM,
        AutoModelForCausalLM,
        AutoModelForQuestionAnswering,
        AutoModelForMaskedLM,
        AutoModel,
        AutoModelForTokenClassification,
        AutoModelForTableQuestionAnswering,
        AutoModelForImageClassification,
    )

if is_tf_available():
    import tensorflow as tf
    from transformers.models.auto.modeling_tf_auto import (
        TFAutoModelForSequenceClassification,
        TFAutoModelForSeq2SeqLM,
        TFAutoModelForCausalLM,
        TFAutoModelForQuestionAnswering,
        TFAutoModelForMaskedLM,
        TFAutoModel,
        TFAutoModelForTokenClassification,
    )

TASKS = {
    "feature-extraction": {
        "impl": FeatureExtractionPipeline,
        "tf": TFAutoModel if is_tf_available() else None,
        "pt": AutoModel if is_torch_available() else None,
        "default": {
            "model": {"pt": "distilbert-base-cased", "tf": "distilbert-base-cased"}
        },
    },
    "text-classification": {
        "impl": TextClassificationPipeline,
        "tf": TFAutoModelForSequenceClassification if is_tf_available() else None,
        "pt": AutoModelForSequenceClassification if is_torch_available() else None,
        "default": {
            "model": {
                "pt": "distilbert-base-uncased-finetuned-sst-2-english",
                "tf": "distilbert-base-uncased-finetuned-sst-2-english",
            },
        },
    },
    "token-classification": {
        "impl": TokenClassificationPipeline,
        "tf": TFAutoModelForTokenClassification if is_tf_available() else None,
        "pt": AutoModelForTokenClassification if is_torch_available() else None,
        "default": {
            "model": {
                "pt": "dbmdz/bert-large-cased-finetuned-conll03-english",
                "tf": "dbmdz/bert-large-cased-finetuned-conll03-english",
            },
        },
    },
    "question-answering": {
        "impl": QuestionAnsweringPipeline,
        "tf": TFAutoModelForQuestionAnswering if is_tf_available() else None,
        "pt": AutoModelForQuestionAnswering if is_torch_available() else None,
        "default": {
            "model": {
                "pt": "distilbert-base-cased-distilled-squad",
                "tf": "distilbert-base-cased-distilled-squad",
            },
        },
    },
    "table-question-answering": {
        "impl": TableQuestionAnsweringPipeline,
        "pt": AutoModelForTableQuestionAnswering if is_torch_available() else None,
        "tf": None,
        "default": {
            "model": {
                "pt": "google/tapas-base-finetuned-wtq",
                "tokenizer": "google/tapas-base-finetuned-wtq",
                "tf": "google/tapas-base-finetuned-wtq",
            },
        },
    },
    "fill-mask": {
        "impl": FillMaskPipeline,
        "tf": TFAutoModelForMaskedLM if is_tf_available() else None,
        "pt": AutoModelForMaskedLM if is_torch_available() else None,
        "default": {"model": {"pt": "distilroberta-base", "tf": "distilroberta-base"}},
    },
    "summarization": {
        "impl": SummarizationPipeline,
        "tf": TFAutoModelForSeq2SeqLM if is_tf_available() else None,
        "pt": AutoModelForSeq2SeqLM if is_torch_available() else None,
        "default": {"model": {"pt": "sshleifer/distilbart-cnn-12-6", "tf": "t5-small"}},
    },
    # This task is a special case as it's parametrized by SRC, TGT languages.
    "translation": {
        "impl": TranslationPipeline,
        "tf": TFAutoModelForSeq2SeqLM if is_tf_available() else None,
        "pt": AutoModelForSeq2SeqLM if is_torch_available() else None,
        "default": {
            ("en", "fr"): {"model": {"pt": "t5-base", "tf": "t5-base"}},
            ("en", "de"): {"model": {"pt": "t5-base", "tf": "t5-base"}},
            ("en", "ro"): {"model": {"pt": "t5-base", "tf": "t5-base"}},
        },
    },
    "text2text-generation": {
        "impl": Text2TextGenerationPipeline,
        "tf": TFAutoModelForSeq2SeqLM if is_tf_available() else None,
        "pt": AutoModelForSeq2SeqLM if is_torch_available() else None,
        "default": {"model": {"pt": "t5-base", "tf": "t5-base"}},
    },
    "text-generation": {
        "impl": TextGenerationPipeline,
        "tf": TFAutoModelForCausalLM if is_tf_available() else None,
        "pt": AutoModelForCausalLM if is_torch_available() else None,
        "default": {"model": {"pt": "gpt2", "tf": "gpt2"}},
    },
    "zero-shot-classification": {
        "impl": ZeroShotClassificationPipeline,
        "tf": TFAutoModelForSequenceClassification if is_tf_available() else None,
        "pt": AutoModelForSequenceClassification if is_torch_available() else None,
        "default": {
            "model": {"pt": "facebook/bart-large-mnli", "tf": "roberta-large-mnli"},
            "config": {"pt": "facebook/bart-large-mnli", "tf": "roberta-large-mnli"},
            "tokenizer": {"pt": "facebook/bart-large-mnli", "tf": "roberta-large-mnli"},
        },
    },
    "conversational": {
        "impl": ConversationalPipeline,
        "tf": TFAutoModelForCausalLM if is_tf_available() else None,
        "pt": AutoModelForCausalLM if is_torch_available() else None,
        "default": {
            "model": {
                "pt": "microsoft/DialoGPT-medium",
                "tf": "microsoft/DialoGPT-medium",
            }
        },
    },
    "image-classification": {
        "impl": ImageClassificationPipeline,
        "tf": None,
        "pt": AutoModelForImageClassification if is_torch_available() else None,
        "default": {"model": {"pt": "google/vit-base-patch16-224"}},
    },
}

TASK_ALIASES = {
    "sentiment-analysis": "text-classification",
    "ner": "token-classification",
}


class NLP(InDriver):
    def get(
        self,
        task: str,
        model: Optional = None,
        config: Optional[Union[str, PretrainedConfig]] = None,
        tokenizer: Optional[Union[str, PreTrainedTokenizer]] = None,
        framework: Optional[str] = None,
        revision: Optional[str] = None,
        model_kwargs: Dict[str, Any] = {},
        **kwargs,
    ) -> Pipeline:
        """
        Args:
        task (:obj:`str`):
            The task defining which pipeline will be returned. Currently accepted tasks are:
                - :obj:`"feature-extraction"`
                - :obj:`"text-classification"`
                - :obj:`"sentiment-analysis"` (alias of :obj:`"text-classification")
                - :obj:`"token-classification"`
                - :obj:`"ner"` (alias of :obj:`"token-classification")
                - :obj:`"question-answering"`
                - :obj:`"fill-mask"`
                - :obj:`"summarization"`
                - :obj:`"translation_xx_to_yy"`
                - :obj:`"translation"`
                - :obj:`"text-generation"`
                - :obj:`"conversational"`
        model (:obj:`str` or :obj:`~transformers.PreTrainedModel` or :obj:`~transformers.TFPreTrainedModel`, `optional`):
            The model that will be used by the pipeline to make predictions.
        config (:obj:`str` or :obj:`~transformers.PretrainedConfig`, `optional`):
            The configuration that will be used by the pipeline to instantiate the model. This can be a model
            identifier or an actual pretrained model configuration inheriting from
            :class:`~transformers.PretrainedConfig`.
        tokenizer (:obj:`str` or :obj:`~transformers.PreTrainedTokenizer`, `optional`):
            The tokenizer that will be used by the pipeline to encode data for the model. This can be a model
            identifier or an actual pretrained tokenizer inheriting from :class:`~transformers.PreTrainedTokenizer`.
         framework (:obj:`str`, `optional`):
            The framework to use, either :obj:`"pt"` for PyTorch or :obj:`"tf"` for TensorFlow. The specified framework
            must be installed.
         kwargs:
            Additional keyword arguments passed along to the specific pipeline init (see the documentation for the
            corresponding pipeline class for possible values).
        Returns:
            NLP Pipeline
        """
        if task in TASK_ALIASES:
            task = TASK_ALIASES[task]

        targeted_task = TASKS[task]
        task_class = targeted_task["impl"]

        if task not in TASKS:
            raise KeyError(
                "Unknown task {}, available tasks are {}".format(
                    task, list(TASKS.keys())
                )
            )

        if framework is None:
            framework, model = infer_framework_from_model(
                model, targeted_task, revision=revision, task=task
            )

        task_class, model_class = targeted_task["impl"], targeted_task[framework]
        if isinstance(config, str):
            config = AutoConfig.from_pretrained(
                config, revision=revision, _from_pipeline=task, **model_kwargs
            )

        if model is None:
            model = targeted_task["default"]["model"][framework]

        if tokenizer is None:
            if isinstance(model, str):
                tokenizer = model
            else:
                # Impossible to guest what is the right tokenizer here
                raise Exception(
                    "Please provided a PretrainedTokenizer "
                    "class or a path/identifier to a pretrained tokenizer."
                )
        if isinstance(tokenizer, (str, tuple)):
            tokenizer = AutoTokenizer.from_pretrained(tokenizer)

        if isinstance(model, str):
            # Handle transparent TF/PT model conversion
            if framework == "pt" and model.endswith(".h5"):
                model_kwargs["from_tf"] = True
                logger.warning(
                    "Model might be a TensorFlow model (ending with `.h5`) but TensorFlow is not available. "
                    "Trying to load the model with PyTorch."
                )
            elif framework == "tf" and model.endswith(".bin"):
                model_kwargs["from_pt"] = True
                logger.warning(
                    "Model might be a PyTorch model (ending with `.bin`) but PyTorch is not available. "
                    "Trying to load the model with Tensorflow."
                )

            if model_class is None:
                raise ValueError(
                    f"Pipeline using {framework} framework, but this framework is not supported by this pipeline."
                )

            model = model_class.from_pretrained(
                model,
                config=config,
                revision=revision,
                _from_pipeline=task,
                **model_kwargs,
            )

        return task_class(
            model=model, framework=framework, tokenizer=tokenizer, task=task, **kwargs
        )



from naas_drivers.driver import InDriver
import plotly.graph_objects as go
import requests
import os


class Plotly(InDriver):
    """Plot generator lib"""

    __css_base = ".modebar {display: none;} \n.modebar-container {display: none;} "

    def __export(self, chart, filename, css=None):
        html_filename = f"{filename.split('.')[0]}.html"
        html_exist = os.path.exists(html_filename)
        chart.write_html(html_filename)
        selector = None
        html_map = None
        if css is None:
            css = self.__css_base
        else:
            css = css + self.__css_base
        with open(html_filename) as f:
            html_map = f.read()
            if html_map.find(" cartesianlayer") != -1:
                selector = ".cartesianlayer"
            result = html_map.replace(
                "</head>", f'<style id="naas_css">{css}</style></head>'
            )
        with open(html_filename, "w") as f:
            f.write(result)
            f.close()
        if filename.endswith(".png") or filename.endswith(".jpeg"):
            html_text = result
            extension = filename.split(".")[1]
            json = {
                "output": "screenshot",
                "html": html_text,
                "emulateScreenMedia": True,
                "ignoreHttpsErrors": True,
                "scrollPage": False,
                "screenshot": {"type": extension},
            }
            if selector:
                json["screenshot"]["selector"] = selector
            req = requests.post(
                url=f"{os.environ.get('SCREENSHOT_API', 'http://naas-screenshot:9000')}/api/render",
                json=json,
            )
            req.raise_for_status()
            open(filename, "wb").write(req.content)
            if not html_exist:
                os.remove(html_filename)
        elif not filename.endswith(".html"):
            print("Not supported for now")
            os.remove(html_filename)
            return
        print(f"Saved as {filename}")

    def export(self, chart, filenames, css=None):
        """create html export and add css to it"""
        if isinstance(filenames, list):
            for filename in filenames:
                self.__export(chart, filename, css)
        else:
            self.__export(chart, filenames, css)

    def __predict(self, stock, visible):
        charts = []
        if "ARIMA" in stock.columns:
            charts.append(
                go.Scatter(
                    visible=visible,
                    x=stock["Date"],
                    y=stock["ARIMA"],
                    mode="lines",
                    name="ARIMA",
                    line=dict(dash="dot"),
                )
            )
        if "COMPOUND" in stock.columns:
            charts.append(
                go.Scatter(
                    visible=visible,
                    x=stock["Date"],
                    y=stock["COMPOUND"],
                    mode="lines",
                    name="COMPOUND",
                    line=dict(dash="dot"),
                )
            )
        if "LINEAR" in stock.columns:
            charts.append(
                go.Scatter(
                    visible=visible,
                    x=stock["Date"],
                    y=stock["LINEAR"],
                    mode="lines",
                    name="LINEAR",
                    line=dict(dash="dot"),
                )
            )
        if "SVR" in stock.columns:
            charts.append(
                go.Scatter(
                    visible=visible,
                    x=stock["Date"],
                    y=stock["SVR"],
                    mode="lines",
                    name="SVR",
                    line=dict(dash="dot"),
                )
            )
        return charts

    def __linechart(self, stock, visible, kind):
        charts = []
        filtered = kind.split("_")[1] if "_" in kind else None
        if "Open" in stock.columns and (filtered is None or filtered == "open"):
            charts.append(
                go.Scatter(
                    visible=visible,
                    x=stock["Date"],
                    y=stock["Open"],
                    mode="lines",
                    name="Open",
                )
            )
        if "Close" in stock.columns and (filtered is None or filtered == "close"):
            charts.append(
                go.Scatter(
                    visible=visible,
                    x=stock["Date"],
                    y=stock["Close"],
                    mode="lines",
                    name="Close",
                )
            )
        return charts

    def __candlestick(self, stock, visible, company):
        return [
            go.Candlestick(
                visible=visible,
                name=company,
                x=stock["Date"],
                open=stock["Open"],
                high=stock["High"],
                low=stock["Low"],
                close=stock["Close"],
            )
        ]

    def __moving_average(self, stock, visible):
        filter_cols = [x for x in stock.columns if x.startswith("MA")]
        if len(filter_cols) == 0:
            return []
        else:
            charts = []
            for i in range(len(filter_cols)):
                filter_col = filter_cols[i]
                line = dict(color="green", width=1) if filter_col == "MA20" else None
                line = dict(color="red", width=1) if filter_col == "MA50" else line
                charts.append(
                    go.Scatter(
                        x=stock["Date"],
                        visible=visible,
                        y=stock[filter_col],
                        line=line,
                        name=f'{filter_col.replace("MA", "")} MA',
                    )
                )
            return charts

    def linechart(
        self,
        dataset,
        label_x: str,
        label_y: list,
        show=False,
    ):
        charts = []
        layout = dict(
            dragmode="pan",
            xaxis_rangeslider_visible=False,
            showlegend=False,
            updatemenus=[],
        )
        for i in range(len(label_y)):
            filter_col = label_y[i]
            charts.append(
                go.Scatter(
                    x=dataset[label_x],
                    y=dataset[filter_col],
                    name=filter_col,
                )
            )
        fig = go.Figure(data=charts, layout=layout)
        if show:
            fig.show()
        return fig

    def candlestick(
        self,
        dataset,
        label_x: str,
        label_open: str,
        label_high: str,
        label_low: str,
        label_close: str,
        show=False,
    ):
        charts = []
        layout = dict(
            dragmode="pan",
            xaxis_rangeslider_visible=False,
            showlegend=False,
            updatemenus=[],
        )
        charts = [
            go.Candlestick(
                x=dataset[label_x],
                open=dataset[label_open],
                high=dataset[label_high],
                low=dataset[label_low],
                close=dataset[label_close],
            )
        ]
        fig = go.Figure(data=charts, layout=layout)
        if show:
            fig.show()
        return fig

    def stock(
        self,
        stock_data,
        kind="candlestick",
        show=False,
        filter=False,
        filter_title="Stock",
        filter_all=False,
    ):
        """generate financial_chart"""
        stock_data_copy = stock_data.copy()
        if "Company" not in stock_data_copy:
            stock_data_copy["Company"] = "Company_1"
        stock_companies = stock_data_copy.Company.unique()
        data = []
        buttons = []
        if filter_all:
            buttons.append(
                dict(
                    args=[{"visible": [True for x in stock_companies]}],
                    label="All",
                    method="restyle",
                )
            )
        for y in range(len(stock_companies)):
            company = stock_companies[y]
            stock = stock_data_copy.loc[stock_data_copy["Company"] == company]
            charts = []
            visible = filter_all if filter_all else y == 0
            charts.extend(self.__moving_average(stock, visible))
            if kind == "candlestick":
                charts.extend(self.__candlestick(stock, visible, company))
            elif kind.startswith("linechart"):
                if any(
                    x in ["ARIMA", "LINEAR", "SVR", "COMPOUND"] for x in stock.columns
                ):
                    charts.extend(self.__predict(stock, visible))
                if any(x in ["Open", "Close"] for x in stock.columns):
                    charts.extend(self.__linechart(stock, visible, kind))
            else:
                print("Not supported for now")
                return
            visibility = []
            for x in stock_companies:
                visibility.extend([x == company] * len(charts))
            data.extend(charts)
            buttons.append(
                dict(
                    args=[{"visible": visibility}],
                    label=company,
                    visible=True,
                    method="restyle",
                )
            )
        print(f"Chart {kind} generated")
        updatemenus = list(
            [
                dict(
                    active=0,
                    buttons=list(buttons),
                    direction="down",
                    pad={"r": 10, "t": 10},
                    showactive=True,
                    x=0.1,
                    xanchor="left",
                    y=1.2,
                    yanchor="top",
                ),
            ]
        )
        layout = None
        if filter:
            layout = dict(
                dragmode="pan",
                xaxis_rangeslider_visible=False,
                showlegend=False,
                updatemenus=updatemenus,
            )
        else:
            layout = dict(
                dragmode="pan",
                xaxis_rangeslider_visible=False,
                showlegend=False,
                updatemenus=[],
            )
        fig = go.Figure(data=list(data), layout=layout)
        if filter:
            fig.update_layout(
                template="plotly_white",
                margin=dict(t=0, b=0, l=0, r=0),
                annotations=[
                    dict(
                        text=filter_title,
                        x=0,
                        xref="paper",
                        font=dict(size=24),
                        y=1.16,
                        yref="paper",
                        align="left",
                        showarrow=False,
                    )
                ],
            )
        if show:
            fig.show()
        return fig

    def table(
        self, header_values, cells_values, header_color="rgb(136,233,175)", show=False
    ):
        """generate table html"""

        fig = go.Figure(
            data=[
                go.Table(
                    header=dict(
                        values=list(header_values),
                        fill_color=header_color,
                        line_color="white",
                        align="center",
                        font=dict(family="Helvetica", color="white", size=14),
                    ),
                    cells=dict(
                        values=cells_values,
                        fill_color="white",
                        line_color="lightgray",
                        align="left",
                        font=dict(family="Helvetica", size=12),
                    ),
                )
            ]
        )

        fig.update_layout(margin=dict(l=10, r=10, t=0, b=0))
        if show:
            fig.show()
        return fig



from naas_drivers.driver import InDriver, OutDriver
import pandas as pd
import requests
import time
from datetime import datetime


class LinkedIn(InDriver, OutDriver):
    # Get LK info from LK
    def __get_id(self, url):
        url = url.rsplit("in/")[-1].rsplit("/")[0]
        return url

    def connect(self, li_at: str, jessionid: str):
        # Init lk attribute
        self.li_at = li_at
        self.jessionid = jessionid

        # Init cookies
        self.cookies = {"li_at": self.li_at, "JSESSIONID": f'"{self.jessionid}"'}

        # Init headers
        self.headers = {
            "X-Li-Lang": "en_US",
            "Accept": "application/vnd.linkedin.normalized+json+2.1",
            "Cache-Control": "no-cache",
            "csrf-Token": self.jessionid.replace('"', ""),
            "X-Requested-With": "XMLHttpRequest",
            "X-Restli-Protocol-Version": "2.0.0",
        }

        # Set connexion to active
        self.connected = True
        return self

    def get_identity(self, username: str):
        username = self.__get_id(username)
        data = requests.get(
            "https://www.linkedin.com/voyager/api/identity/profiles/"
            + username.replace("\n", ""),
            cookies=self.cookies,
            headers=self.headers,
        )
        return data.json()

    def get_network(self, username: str):
        username = self.__get_id(username)
        data = requests.get(
            "https://www.linkedin.com/voyager/api/identity/profiles/"
            + username
            + "/networkinfo",
            cookies=self.cookies,
            headers=self.headers,
        )
        return data.json()

    def get_contact(self, username: str):
        username = self.__get_id(username)
        data = requests.get(
            "https://www.linkedin.com/voyager/api/identity/profiles/"
            + username.replace("\n", "")
            + "/profileContactInfo",
            cookies=self.cookies,
            headers=self.headers,
        )
        return data.json()

    def get_profil(self, username: str, output="dataframe"):
        username = self.__get_id(username)
        # Get data from identity
        time.sleep(2)
        profil = self.get_identity(username)
        time.sleep(2)
        network = self.get_network(username)
        time.sleep(2)
        contact = self.get_contact(username)

        # Get profil info
        firstname = None
        lastname = None
        country = None
        adress = None
        lk_headline = None
        lk_industry = None
        bd = None
        bd_day = None
        bd_month = None
        bd_year = None

        pf = profil.get("data")
        if pf is not None:
            firstname = pf.get("firstName")
            lastname = pf.get("lastName").upper()
            birthdate = pf.get("birthDate")
            if birthdate is not None:
                bd_day = birthdate.get("day", "Day Unknown")
                bd_month = birthdate.get("month", "Month Unknown")
                bd_year = birthdate.get("year", "Year Unknown")
                bd = f"{bd_day}, {bd_month} - {bd_year}"
            country = pf.get("geoCountryName")
            adress = pf.get("geoLocationName")
            lk_headline = pf.get("headline")
            lk_industry = pf.get("industryName")

        # Get network info
        nw = network.get("data")
        lk_followers = None
        if nw is not None:
            lk_followers = nw.get("followersCount")

        # Get contact info
        ct = contact.get("data")
        lk_phone = None
        lk_email = None
        lk_twiter = None
        if ct is not None:
            lk_phone = None
            lk_phones = ct.get("phoneNumbers")
            if lk_phones is not None:
                for rows in lk_phones:
                    if rows["type"] == "MOBILE":
                        lk_phone = rows["number"]
                        break
            lk_email = ct.get("emailAddress")
            lk_twiter = None
            lk_twiters = ct.get("twitterHandles")
            if lk_twiters is not None:
                for rows in lk_twiters:
                    lk_twiter = rows["name"]
                    break

        # Profile dict
        lk_profile = {
            "FIRSTNAME": firstname,
            "LASTNAME": lastname,
            "BIRTHDATE_DAY": bd_day,
            "BIRTHDATE_MONTH": bd_month,
            "BIRTHDATE_YEAR": bd_year,
            "BIRTHDATE": bd,
            "COUNTRY": country,
            "ADDRESS": adress,
            "LK_HEADLINE": lk_headline,
            "LK_SECTOR": lk_industry,
            "LK_FOLLOWERS": lk_followers,
            "LK_PHONE": lk_phone,
            "LK_EMAIL": lk_email,
            "LK_TWITER": lk_twiter,
        }

        if output == "json":
            return lk_profile

        if output == "dataframe":
            df = pd.DataFrame.from_records([lk_profile])
            return df

    def get_conversations(self):
        data = requests.get(
            "https://www.linkedin.com/voyager/api/messaging/conversations",
            cookies=self.cookies,
            headers=self.headers,
        )
        return data.json()

    def get_messages(self):
        # Get lk conversation
        message = self.get_conversations()

        # Transform conversation
        messages = message.get("included")
        lk_profile = []
        lk_conversation = []
        lk_event = []
        for m in messages:
            lk_type = m.get("$type")
            if lk_type == "com.linkedin.voyager.identity.shared.MiniProfile":
                # Init variable
                firstname = None
                lastname = None
                occupation = None
                publicidentifier = None
                profile_id = None

                # Get variable from dict
                firstname = m.get("firstName")
                lastname = m.get("lastName")
                occupation = m.get("occupation")
                publicidentifier = m.get("publicIdentifier")
                profile_id = m.get("entityUrn")
                profile_id = profile_id.rsplit("urn:li:fs_miniProfile:")[-1]

                # Create profile dict
                if profile_id != "UNKNOWN":
                    tmp_dict = {}
                    tmp_dict = {
                        "FIRSTNAME": firstname,
                        "LASTNAME": lastname,
                        "OCCUPATION": occupation,
                        "PROFILE_ID": profile_id,
                        "PROFILE_PUBLIC_ID": publicidentifier,
                    }
                    lk_profile.append(tmp_dict)

            if lk_type == "com.linkedin.voyager.messaging.Conversation":
                # Init variable
                profile_id = None
                message_id = None
                lastactivityat = None
                lastreadat = None

                # Get variable from dict
                profile_id = m.get("*participants")
                message_id = m.get("entityUrn")
                message_id = message_id.rsplit("urn:li:fs_conversation:")[-1]
                lastactivityat = m.get("lastActivityAt")
                lastreadat = m.get("lastReadAt")
                if lastactivityat is not None:
                    lastactivityat = datetime.fromtimestamp(lastactivityat / 1000.0)
                if lastreadat is not None:
                    lastreadat = datetime.fromtimestamp(lastreadat / 1000.0)

                # Create conversation dict
                profile_id = str(profile_id).rsplit(",")[-1].rsplit(")")[0]
                if profile_id != "UNKNOWN":
                    tmp_dict = {}
                    tmp_dict = {
                        "PROFILE_ID": profile_id,
                        "MESSAGE_ID": message_id,
                        "LAST_ACTIVITY": lastactivityat,
                        "LAST_READ_AT": lastreadat,
                    }
                    lk_conversation.append(tmp_dict)

            if lk_type == "com.linkedin.voyager.messaging.Event":
                # Init variable
                message_id = None
                message_type = None
                message_text = None

                # Get variable from dict
                message_id = m.get("entityUrn")
                message_id = message_id.rsplit("urn:li:fs_event:(")[-1].rsplit(",")[0]
                message_type = m.get("subtype")
                if message_type != "SPONSORED_INMAIL":
                    for key in m.get("eventContent"):
                        if key == "attributedBody":
                            message_text = m.get("eventContent").get(key).get("text")
                            break

                # Create event dict
                tmp_dict = {}
                tmp_dict = {
                    "MESSAGE_ID": message_id,
                    "MESSAGE_TEXT": message_text,
                    "MESSAGE_TYPE": message_type,
                }
                lk_event.append(tmp_dict)

        # Convert dict to dataframe
        df_profile = pd.DataFrame.from_records(lk_profile)
        df_conversation = pd.DataFrame.from_records(lk_conversation)
        df_event = pd.DataFrame.from_records(lk_event)

        # Merge dataframe
        df_message = pd.merge(df_profile, df_conversation, on="PROFILE_ID", how="left")

        df_message = pd.merge(df_message, df_event, on="MESSAGE_ID", how="left")
        # Cleaning
        to_drop = ["MESSAGE_ID", "PROFILE_ID"]
        df_message = df_message.drop(to_drop, axis=1)
        return df_message.reset_index(drop=True)

    def get_post(self, url):
        activity_id = url.split("activity-")[-1].split("-")[0]
        data = requests.get(
            f"https://www.linkedin.com/voyager/api/feed/updates/urn:li:activity:{activity_id}",
            cookies=self.cookies,
            headers=self.headers,
        )
        return data.json()

    def get_post_data(self, url):
        activity_id = url.split("activity-")[-1].split("-")[0]
        # Get lk conversation
        post = self.get_post(url)

        # Init var
        title = None
        datepost = None
        tot_views = 0
        tot_comments = 0
        tot_likes = 0
        num_lik = 0
        num_pra = 0
        num_int = 0
        num_app = 0
        num_emp = 0

        # Parse json
        posts = post.get("included")
        if posts is not None:
            for p in posts:
                lk_type = p.get("$type")
                if lk_type == "com.linkedin.voyager.feed.shared.SocialActivityCounts":
                    uid = p.get("entityUrn")
                    if (
                        uid
                        == f"urn:li:fs_socialActivityCounts:urn:li:activity:{activity_id}"
                        or "urn:li:fs_socialActivityCounts:urn:li:ugcPost" in uid
                    ):
                        tot_likes = p.get("numLikes")
                        tot_views = p.get("numViews")
                        tot_comments = p.get("numComments")
                        likes = p.get("reactionTypeCounts")
                        if likes is not None:
                            for like in likes:
                                reaction = like.get("reactionType")
                                if reaction == "LIKE":
                                    num_lik = like.get("count")
                                if reaction == "PRAISE":
                                    num_pra = like.get("count")
                                if reaction == "INTEREST":
                                    num_int = like.get("count")
                                if reaction == "APPRECIATION":
                                    num_app = like.get("count")
                                if reaction == "EMPATHY":
                                    num_emp = like.get("count")
                if lk_type == "com.linkedin.voyager.feed.render.UpdateV2":
                    commentary = p.get("commentary")
                    if commentary is not None:
                        title = commentary.get("text").get("text").rsplit("\n")[0]
                    datepost = (
                        p.get("actor").get("subDescription").get("accessibilityText")
                    )

        # Data
        data = {
            "URL": url,
            "TITLE": title,
            "DATE": datepost,
            "VIEWS": tot_views,
            "COMMENTS": tot_comments,
            "LIKES": tot_likes,
            "LIKES_LIKE": num_lik,
            "LIKES_PRAISE": num_pra,
            "LIKES_INTEREST": num_int,
            "LIKES_APPRECIATION": num_app,
            "LIKES_EMPATHY": num_emp,
        }

        # DataFrame
        df = pd.DataFrame([data])
        return df



from naas_drivers.driver import InDriver
import pandas as pd
import requests
import os
from datetime import datetime

DATE_FORMAT = "%Y-%m-%d"


class Organizations:
    def __init__(self, user_id, api_key):
        self.base_url = os.environ.get(
            "QONTO_API_URL", "https://thirdparty.qonto.eu/v2"
        )
        self.req_headers = {"authorization": f"{user_id}:{api_key}"}
        self.url = f"{self.base_url}/organizations"
        self.user_id = user_id

    def get(self):
        try:
            req = requests.get(
                url=f"{self.url}/{self.user_id}", headers=self.req_headers
            )
            req.raise_for_status()
            items = req.json()["organization"]["bank_accounts"]
            df = pd.DataFrame.from_records(items)

            # Formating CS
            df["date"] = datetime.now().strftime("%d/%m/%Y %H:%M:%S")
            df = df.drop(["slug", "balance_cents", "authorized_balance_cents"], axis=1)
            df.columns = df.columns.str.upper()
            return df
        except requests.HTTPError as err:
            err_code = err.response.status_code
            err_msg = err.response.json()
            to_print = f"{err_code}: {err_msg}"
            print(to_print)


class Transactions(Organizations):
    def get_all(self):
        # Get organizations
        df_organisations = self.get()

        # For each bank account, get all transactions
        df_transaction = pd.DataFrame()
        for _, row in df_organisations.iterrows():
            iban = row["IBAN"]

            # Get transactions
            current_page = "1"
            has_more = True
            while has_more:
                req = requests.get(
                    url=f"{self.base_url}/transactions?current_page={current_page}?per_page=100&iban={iban}",
                    headers=self.req_headers,
                )
                items = req.json()
                transactions = items["transactions"]
                df = pd.DataFrame.from_records(transactions)
                df["iban"] = iban
                df_transaction = pd.concat([df_transaction, df], axis=0)
                # Check if next page exists
                next_page = items["meta"]["next_page"]
                if next_page is None:
                    has_more = False
                else:
                    current_page = str(next_page)
        # Formatting
        to_keep = [
            "iban",
            "settled_at",
            "emitted_at",
            "transaction_id",
            "label",
            "reference",
            "operation_type",
            "side",
            "amount",
            "currency",
        ]
        df_transaction = (
            df_transaction[to_keep].reset_index(drop=True).fillna("Not affected")
        )
        df_transaction.loc[
            df_transaction["side"] == "debit", "amount"
        ] = df_transaction["amount"] * (-1)
        df_transaction.columns = df_transaction.columns.str.upper()
        return df_transaction


class Statements(Transactions):
    def __filter_dates(self, df, date_from=None, date_to=None):
        # Dates
        if date_from is not None and date_to is None:
            date_to = df["DATE"].max()

        if date_to is not None and date_from is None:
            date_from = df["DATE"].min()

        if (date_from and date_to) is not None:
            dates_range = pd.date_range(start=date_from, end=date_to)
            dates = []
            for date in dates_range:
                date = str(date.strftime(DATE_FORMAT))
                dates.append(date)

            df = df[df["DATE"].isin(dates)]
        return df

    def detailed(self, date_from=None, date_to=None):
        df = self.get_all()
        df = df.rename(columns={"EMITTED_AT": "DATE"})
        df["DATE"] = pd.to_datetime(
            df["DATE"], format="%Y-%m-%dT%H:%M:%S.%fZ"
        ).dt.strftime(DATE_FORMAT)

        # Calc positions
        to_sort = ["IBAN", "DATE"]
        df = df.sort_values(by=to_sort).reset_index(drop=True)
        to_group = ["IBAN"]
        df["POSITION"] = df.groupby(to_group, as_index=True).agg({"AMOUNT": "cumsum"})
        to_keep = [
            "IBAN",
            "DATE",
            "TRANSACTION_ID",
            "LABEL",
            "REFERENCE",
            "OPERATION_TYPE",
            "AMOUNT",
            "POSITION",
            "CURRENCY",
        ]
        df = df[to_keep]
        return self.__filter_dates(df, date_from, date_to)

    def aggregated(self, date_from=None, date_to=None):
        df = self.get_all()
        df = df.rename(columns={"EMITTED_AT": "DATE"})
        df["DATE"] = pd.to_datetime(
            df["DATE"], format="%Y-%m-%dT%H:%M:%S.%fZ"
        ).dt.strftime(DATE_FORMAT)

        # Aggregation
        to_group = ["IBAN", "DATE", "CURRENCY"]
        df = df.groupby(to_group, as_index=False).agg({"AMOUNT": "sum"})

        # Calc positions
        to_sort = ["IBAN", "DATE"]
        df = df.sort_values(by=to_sort).reset_index(drop=True)
        to_group = ["IBAN"]
        df["POSITION"] = df.groupby(to_group, as_index=True).agg({"AMOUNT": "cumsum"})
        to_keep = [
            "IBAN",
            "DATE",
            "AMOUNT",
            "POSITION",
            "CURRENCY",
        ]
        df = df[to_keep]
        return self.__filter_dates(df, date_from, date_to)


class Qonto(InDriver):
    user_id = None
    api_token = None

    def connect(self, user_id, api_token):
        # Init thinkific attribute
        self.user_id = user_id
        self.token = api_token

        # Init end point
        self.positions = Organizations(self.user_id, self.token)
        self.flows = Transactions(self.user_id, self.token)
        self.statement = Statements(self.user_id, self.token)

        # Set connexion to active
        self.connected = True
        return self



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="naas-drivers",
    version="0.57.1",
    author="Martin Donadieu",
    author_email="martindonadieu@gmail.com",
    license="BSD",
    description="Drivers made to easy connect to any services",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jupyter-naas/drivers",
    packages=find_packages(exclude=["tests"]),
    extras_require={
        "dev": [
            "pytest==6.2.3",
            "pytest-mock==3.6.0",
            "requests-mock==1.9.1",
            "twine==3.4.1",
            "flake8==3.9.1",
            "black==21.4b2",
            "commitizen==2.17.4",
            "pytest-cov==2.11.1",
        ]
    },
    install_requires=[
        "pyppeteer==0.2.5",
        "imap_tools==0.39.0",
        "slackclient==2.9.3",
        "pymsteams==0.1.14",
        "pdfkit==0.6.1",
        "markdown2==2.4.0",
        "newsapi-python==0.2.6",
        "airtable-python-wrapper==0.15.2",
        "notion==0.0.28",
        "pyjwt==2.1.0",
        "tensorflow==2.4.1",
        "pysftp==0.2.9",
        "htmlbuilder==0.1.2",
        "vaderSentiment==3.3.2",
        "chardet==4.0.0",
        "Cython==0.29.23",
        "idna==2.9",
        "inflection==0.5.1",
        "joblib==1.0.1",
        "more-itertools==8.7.0",
        "numpy==1.19.5",
        "ipython==7.22.0",
        "pandas==1.2.4",
        "pandas-datareader==0.9.0",
        "patsy==0.5.1",
        "pmdarima==1.8.2",
        "python-dateutil==2.8.1",
        "python-dotenv==0.17.0",
        "pytz==2021.1",
        "plotly==4.14.3",
        "kaleido==0.2.1",
        "Quandl==3.6.1",
        "requests==2.25.1",
        "scikit-learn==0.24.2",
        "torch==1.8.1",
        "scipy==1.6.3",
        "six==1.15.0",
        "statsmodels==0.12.2",
        "urllib3==1.26.4",
        "xlrd==2.0.1",
        "pymongo==3.11.3",
        "pysftp==0.2.9",
        "md2pdf==0.5",
        "sendgrid==6.7.0",
        "escapism==1.0.1",
        "openpyxl==3.0.7",
        "google==3.0.0",
        "google-api-python-client==2.3.0",
        "google-auth-httplib2==0.1.0",
        "google-auth-oauthlib==0.4.4",
        "gspread==3.7.0",
        "oauth2client==4.1.3",
        "geopy==2.1.0",
        "GitPython==3.1.14",
        "cson==0.8",
        "opencv-python==4.5.1.48",
        "pytesseract==0.3.7",
        "wkhtmltopdf==0.2",
        "streamlit==0.82.0",
        "transformers==4.6.1",
        "pyngrok==5.0.5",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: BSD License",
        "Framework :: Jupyter",
        "Operating System :: OS Independent",
    ],
)



from naas_drivers.driver import InDriver, OutDriver
from urllib.parse import urljoin
import pandas as pd
import requests
import os


BIG_NUM_TO_GETALL = 1000000


class Gsheet(InDriver, OutDriver):
    sheets_api = None
    spreadsheet_id = None
    sheet_name = None

    def __init__(self):
        self.sheets_api = os.getenv("GSHEETS_API")

    def connect(
        self,
        spreadsheet_id: str,
        api_url: str = None,
    ):
        self.spreadsheet_id = spreadsheet_id
        self.sheets_api = api_url if api_url else os.getenv("GSHEETS_API")
        self.connected = True
        return self

    def delete(
        self,
        sheet_name: str,
        rows: list = [],
    ):
        self.check_connect()
        resp = requests.delete(
            urljoin(self.sheets_api, f"{self.spreadsheet_id}/{sheet_name}"), json=rows
        )
        data = resp.json()
        if data.get("error"):
            self.print_error(data.get("error"))
        return data

    def get(
        self,
        sheet_name: str,
        items_per_page: int = BIG_NUM_TO_GETALL,
    ) -> pd.DataFrame:
        self.check_connect()
        resp = requests.get(
            urljoin(self.sheets_api, f"{self.spreadsheet_id}/{sheet_name}"),
            params={"perPage": items_per_page},
        )
        data = resp.json()
        if data.get("error"):
            self.print_error(data.get("error"))
        df = pd.DataFrame(data=data["data"], columns=data["columns"])
        return df

    def send(self, data, sheet_name: str, append: bool = True) -> str:
        self.check_connect()
        data_formated = data
        if isinstance(data, pd.DataFrame):
            for column in data:
                if data[column].dtypes == "datetime64[ns]":
                    data[column] = data[column].astype(str)
            data_formated = data.to_dict(orient="records")

        if not append:
            try:
                resp = requests.get(
                    urljoin(self.sheets_api, f"{self.spreadsheet_id}/{sheet_name}"),
                    params={"perPage": BIG_NUM_TO_GETALL},
                )
                resp.raise_for_status()
                data = resp.json()
                if data.get("data"):
                    row_count = data.get("data")[0].get("rowNumber")
                    df = pd.DataFrame(data=data["data"], columns=data["columns"])
                    row_count = len(df)
                    requests.delete(
                        urljoin(self.sheets_api, f"{self.spreadsheet_id}/{sheet_name}"),
                        json=list(range(1, row_count + 2)),
                    )
            except Exception:
                pass
        resp = requests.post(
            urljoin(self.sheets_api, f"{self.spreadsheet_id}/{sheet_name}"),
            json=data_formated,
        )
        data = resp.json()
        if data.get("error"):
            self.print_error(data.get("error"))
        return data
